using System;
using System.Data;
using System.Collections;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using System.IO;

using CMS.DataEngine;
using CMS.GlobalHelper;
using CMS.URLRewritingEngine;
using CMS.UIControls;
using CMS.Controls;
using CMS.ExtendedControls;

public partial class CMSAdminControls_UI_UniGrid_UniGrid : UniGrid
{
    #region "Variables"

    private Button showButton = null;
    private bool regularPaging = false;
    private int rowIndex = 1;
    private bool resetSelection = false;

    #endregion


    #region "Properties"

    /// <summary>
    /// Gets or sets GridView control of UniGrid
    /// </summary>
    public override GridView GridView
    {
        get
        {
            return UniGridView;
        }

        set
        {
            UniGridView = value;
        }
    }


    /// <summary>
    /// Gets selected items from UniGrid.
    /// </summary>
    public override ArrayList SelectedItems
    {
        get
        {
            return GetSelectedItems();
        }
        set
        {
            SetSectedItems(value);
        }
    }


    /// <summary>
    /// Gets selected items from UniGrid.
    /// </summary>
    public override ArrayList DeselectedItems
    {
        get
        {
            return GetDeselectedItems();
        }
    }


    /// <summary>
    /// Gets selected items from UniGrid.
    /// </summary>
    public override ArrayList NewlySelectedItems
    {
        get
        {
            return GetNewlySelectedItems();
        }
    }


    /// <summary>
    /// Gets page size dropdown from Unigrid.
    /// </summary>
    public override DropDownList PageSizeDropdown
    {
        get
        {
            return pageSizeDropdown;
        }
    }


    /// <summary>
    /// Gets filter placeHolder from Unigrid.
    /// </summary>
    public override PlaceHolder FilterPlaceHolder
    {
        get
        {
            return filter;
        }
    }

    #endregion


    #region "Methods"

    /// <summary>
    /// Control load event handler
    /// </summary>
    protected void Page_Load(object sender, EventArgs e)
    {
        if (StopProcessing)
        {
            Visible = false;
        }
        else
        {
            if (ImageDirectoryPath == null)
            {
                ImageDirectoryPath = GetImageUrl("Design/Controls/UniGrid/Actions", IsLiveSite, true);
            }

            lblItems.Text = ResHelper.GetString("UniGrid.ItemsPerPage");

            OnBeforePageIndexChanging += UniGridView_OnBeforePageIndexChanging;

            if (LoadGridDefinition())
            {
                ActionsHash = hidActionsHash;
                // Check whether current request is row action command and if so, raise action
                if (!String.IsNullOrEmpty(hidCmdName.Value) && (Request.Form["__EVENTTARGET"] == UniqueID) && ((Request.Form["__EVENTARGUMENT"] == "UniGridAction")))
                {
                    RaiseAction(hidCmdName.Value, hidCmdArg.Value);
                }

                // Set order by clause
                ProcessSorting();
                // Set page size dropdown
                SetPageSize();
                // Get data from database and set them to the grid view
                if (FilterByQueryString)
                {
                    if (displayFilter)
                    {
                        SetFilter(true);
                    }
                    else
                    {
                        if (!EventRequest() && !DelayedReload)
                        {
                            ReloadData();
                        }
                    }
                }
                else
                {
                    if (!EventRequest() && !DelayedReload)
                    {
                        ReloadData();
                    }
                }
            }
        }

        // Clear hidden action on load event. If unigrid is invisible, page pre render is not fired
        ClearActions();
    }


    /// <summary>
    /// Control PreRender event handler
    /// </summary>
    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        if (Visible && !StopProcessing)
        {
            RegisterCmdScripts();
        }

        if (pageSizeDropdown.Items.Count <= 1)
        {
            pnlFooter.Visible = false;
        }

        if (FilterIsSet)
        {
            // Check for FilteredZeroRowsText
            if ((GridView.Rows.Count == 0) && !String.IsNullOrEmpty(FilteredZeroRowsText))
            {
                // Display filter zero rows text
                lblInfo.Text = FilteredZeroRowsText;
                lblInfo.Visible = true;
            }
            else
            {
                lblInfo.Visible = false;
            }
        }
        else
        {
            // Check for ZeroRowsText
            if (!HideControlForZeroRows && (GridView.Rows.Count == 0) && !String.IsNullOrEmpty(ZeroRowsText))
            {
                // Display zero rows text
                lblInfo.Text = ZeroRowsText;
                lblInfo.Visible = true;
            }
            else
            {
                lblInfo.Visible = false;
            }
        }
    }


    /// <summary>
    /// Sets the sort direction if current request is sorting
    /// </summary>
    private void ProcessSorting()
    {
        // Get current event target
        string uniqieId = ValidationHelper.GetString(Request.Params["__EVENTTARGET"], String.Empty);
        // Get current argument
        string eventargument = ValidationHelper.GetString(Request.Params["__EVENTARGUMENT"], String.Empty);

        if (uniqieId == GridView.UniqueID)
        {
            if (eventargument.StartsWith("Sort"))
            {
                string orderByColumn = Convert.ToString(eventargument.Split('$')[1]);

                // If sorting is called for the first time and default sorting (OrderBy property) is set
                if ((SortDirect == "") && !string.IsNullOrEmpty(OrderBy))
                {
                    ChangeSortDirection(orderByColumn, OrderBy);
                }
                else
                {
                    ChangeSortDirection(orderByColumn, SortDirect);
                }
            }
        }
    }


    /// <summary>
    /// Returns true if current request was fired by page change or filter show button
    /// </summary>
    private bool EventRequest()
    {
        if (UrlHelper.IsPostback())
        {
            // Get current event target
            string uniqieId = ValidationHelper.GetString(Request.Params["__EVENTTARGET"], String.Empty);
            // Get current argument
            string eventargument = ValidationHelper.GetString(Request.Params["__EVENTARGUMENT"], String.Empty).ToLower();

            // Check whether current request is paging
            if (!String.IsNullOrEmpty(uniqieId) && (uniqieId == GridView.UniqueID) && eventargument.StartsWith("page"))
            {
                return true;
            }

            // Check whether show button is defined
            if (showButton != null)
            {
                // If button name is not empty => button fire postback
                if (!string.IsNullOrEmpty(Request.Params[showButton.UniqueID]))
                {
                    return true;
                }
            }
        }

        // Non-paging request by default
        return false;
    }


    /// <summary>
    /// Returns icon file for current theme or from default if current doesn't exist
    /// </summary>
    /// <param name="iconfile">Icon file name</param>
    private string GetActionImage(string iconfile)
    {
        if (File.Exists(MapPath(ImageDirectoryPath + iconfile)))
        {
            return (ImageDirectoryPath + iconfile);
        }

        return GetImageUrl("Design/Controls/UniGrid/Actions/" + iconfile);
    }


    /// <summary>
    /// Clears UniGrid's information on recently performed action. Under normal circumstances there is no need to perform this action.
    /// However sometimes forcing grid to clear the actions is required.
    /// </summary>
    public void ClearActions()
    {
        // Clear hiddden fields
        hidCmdName.Value = null;
        hidCmdArg.Value = null;
    }


    /// <summary>
    /// Loads the grid definition
    /// </summary>
    public bool LoadGridDefinition()
    {
        if (string.IsNullOrEmpty(GridName))
        {
            return false;
        }
        string xmlFilePath = Server.MapPath(GridName);

        // Check the configuration file
        if (!File.Exists(xmlFilePath))
        {
            lblError.Text = String.Format(ResHelper.GetString("unigrid.noxmlfile"), xmlFilePath);
            lblError.Visible = true;
            return false;
        }

        Table filterTable = new Table();
        XmlDocument document = new XmlDocument();
        document.Load(xmlFilePath);
        XmlNode node = document.DocumentElement;

        UniGridView.Columns.Clear(); // Clear all columns from the grid view
        UniGridView.GridLines = GridLines.Horizontal;

        if (node != null)
        {
            XmlNode optionNode = node.SelectSingleNode("options");

            if (GridName != null)
            {
                // Create filter table according to the key value "DisplayFilter"            
                displayFilter = (GetKeyValue(optionNode, "DisplayFilter").ToLower() == "true") ? true : false;
                if (displayFilter)
                {
                    filter.Controls.Add(filterTable);
                }

                // Filter limit
                int filterLimit = ValidationHelper.GetInteger(GetKeyValue(optionNode, "FilterLimit"), -1);
                if (filterLimit > -1)
                {
                    FilterLimit = filterLimit;
                }

                // Process 'template field'
                // --------------------------------------------------
                XmlNode actions = node.SelectSingleNode("actions");
                if (actions != null)
                {
                    TemplateField ckhColumn = new TemplateField(); // Custom template field of the grid view

                    // Ensure width of the column
                    if (actions.Attributes["width"] != null)
                    {
                        ckhColumn.ItemStyle.Width = new Unit(actions.Attributes["width"].Value);
                    }

                    // Show header?
                    XmlAttribute showHeader = actions.Attributes["showheader"];
                    if ((showHeader == null) || (showHeader.Value.ToLower() == "true"))
                    {
                        // Fill in the custom template field
                        ckhColumn.HeaderTemplate = new GridViewTemplate(ListItemType.Header, ResHelper.GetString("unigrid.actions"));
                    }

                    GridViewTemplate actionsTemplate = new GridViewTemplate(ListItemType.Item, actions, ImageDirectoryPath, DefaultImageDirectoryPath);
                    actionsTemplate.Page = Page;
                    ckhColumn.ItemTemplate = actionsTemplate;

                    ckhColumn.ItemStyle.Wrap = false;

                    // Add custom column to grid view
                    UniGridView.Columns.Add(ckhColumn);

                    actionsTemplate.OnExternalDataBound += RaiseExternalDataBound;
                }

                // Display sort direction images
                string showDirection = GetKeyValue(optionNode, "ShowSortDirection");
                if ((showDirection != "") && (showDirection.ToLower() == "false"))
                {
                    showSortDirection = false;
                }
                else
                {
                    showSortDirection = true;
                }

                // Display selection column with checkboxes
                showSelection = (GetKeyValue(optionNode, "ShowSelection").ToLower() == "true") ? true : false;
                if (showSelection)
                {
                    TemplateField chkColumn = new TemplateField();

                    CheckBox headerBox = new CheckBox();
                    headerBox.ID = "headerBox";

                    CheckBox itemBox = new CheckBox();
                    itemBox.ID = "itemBox";
                    // Set selection argument
                    itemBox.Attributes["selectioncolumn"] = GetKeyValue(optionNode, "SelectionColumn");

                    GridViewTemplate headerTemplate = new GridViewTemplate(ListItemType.Header, headerBox);
                    GridViewTemplate itemTemplate = new GridViewTemplate(ListItemType.Item, itemBox);

                    chkColumn.HeaderTemplate = headerTemplate;
                    chkColumn.ItemTemplate = itemTemplate;
                    UniGridView.Columns.Add(chkColumn);
                }

                // Get pagesize options
                if (GetKeyValue(optionNode, "PageSize") != "")
                {
                    PageSize = GetKeyValue(optionNode, "PageSize");
                }

                // Process 'bound fields'
                // --------------------------------------------------

                // Select list of "column" nodes
                XmlNodeList columnList = node.SelectSingleNode("columns").SelectNodes("column");

                // Get attributes AND filter node of each "column" node
                if (columnList != null)
                {
                    foreach (XmlNode columnNode in columnList)
                    {
                        string columnSource = null, columnCaption = null; // Attributes of the "column" node
                        DataControlField field = null;

                        XmlAttribute href = columnNode.Attributes["href"];
                        XmlAttribute externalSourceName = columnNode.Attributes["externalsourcename"];
                        XmlAttribute localize = columnNode.Attributes["localize"];
                        XmlAttribute icon = columnNode.Attributes["icon"];
                        XmlNode tooltipNode = columnNode.SelectSingleNode("tooltip");
                        XmlAttribute action = columnNode.Attributes["action"];
                        XmlAttribute maxlength = columnNode.Attributes["maxlength"];

                        // Process the column type Hyperlink or BoundColumn based on the parameters
                        if ((href != null) || (externalSourceName != null) || (localize != null) || (icon != null) || (tooltipNode != null) || (action != null) || (maxlength != null))
                        {
                            ExtendedBoundField linkColumn = new ExtendedBoundField();
                            field = linkColumn;

                            // Attribute "source"
                            XmlAttribute source = columnNode.Attributes["source"];
                            if (source != null)
                            {
                                columnSource = source.Value;
                                linkColumn.DataField = columnSource;
                                XmlAttribute allowSorting = columnNode.Attributes["allowsorting"];
                                if ((allowSorting == null) || ValidationHelper.GetBoolean(allowSorting.Value, true))
                                {
                                    if (columnSource.ToLower() != ExtendedBoundField.ALL_DATA.ToLower())
                                    {
                                        linkColumn.SortExpression = columnSource;
                                    }
                                    else
                                    {
                                        // Sorting
                                        XmlAttribute sort = columnNode.Attributes["sort"];
                                        if (sort != null)
                                        {
                                            linkColumn.SortExpression = sort.Value;
                                        }
                                    }
                                }
                            }

                            // Action parameters
                            if (action != null)
                            {
                                linkColumn.Action = action.Value;

                                // Action parameters
                                XmlAttribute commandArgument = columnNode.Attributes["commandargument"];
                                if (commandArgument != null)
                                {
                                    linkColumn.CommandArgument = commandArgument.Value;
                                }
                            }

                            // Action parameters
                            XmlAttribute parameters = columnNode.Attributes["parameters"];
                            if (parameters != null)
                            {
                                linkColumn.ActionParameters = parameters.Value;
                            }

                            // Navigate URL
                            if (href != null)
                            {
                                linkColumn.NavigateUrl = href.Value;
                            }

                            // External source
                            if (externalSourceName != null)
                            {
                                linkColumn.ExternalSourceName = externalSourceName.Value;
                                linkColumn.OnExternalDataBound += RaiseExternalDataBound;
                            }

                            // Localize strings?
                            if (localize != null)
                            {
                                linkColumn.LocalizeStrings = ValidationHelper.GetBoolean(localize.Value, false);
                            }

                            // Style
                            XmlAttribute style = columnNode.Attributes["style"];
                            if (style != null)
                            {
                                linkColumn.Style = style.Value;
                            }

                            // Icon
                            if (icon != null)
                            {
                                if (linkColumn.DataField == "")
                                {
                                    linkColumn.DataField = ExtendedBoundField.ALL_DATA;
                                }
                                linkColumn.Icon = GetActionImage(icon.Value);
                            }

                            // Max length
                            if (maxlength != null)
                            {
                                linkColumn.MaxLength = ValidationHelper.GetInteger(maxlength.Value, 0);
                            }

                            // Process "tooltip" node
                            if (tooltipNode != null)
                            {
                                XmlAttribute tooltipSource = tooltipNode.Attributes["source"];
                                XmlAttribute tooltipExternalSource = tooltipNode.Attributes["externalsourcename"];
                                XmlAttribute tooltipWidth = tooltipNode.Attributes["width"];
                                XmlAttribute tooltipEncode = tooltipNode.Attributes["encode"];

                                // If there is some tooltip register TooltipScript
                                if ((tooltipSource != null) || (tooltipExternalSource != null))
                                {
                                    ScriptHelper.RegisterTooltip(Page);
                                }

                                // Tooltip source
                                if (tooltipSource != null)
                                {
                                    linkColumn.TooltipSourceName = tooltipSource.Value;
                                }

                                // Tooltip external source
                                if (tooltipExternalSource != null)
                                {
                                    linkColumn.TooltipExternalSourceName = tooltipExternalSource.Value;
                                }

                                // Tooltip width
                                if (tooltipWidth != null)
                                {
                                    linkColumn.TooltipWidth = tooltipWidth.Value;
                                }

                                // Encode tooltip
                                if (tooltipEncode != null)
                                {
                                    linkColumn.TooltipEncode = ValidationHelper.GetBoolean(tooltipEncode.Value, true);
                                }
                            }
                        }
                        else
                        {
                            BoundField userColumn = new BoundField(); // Custom column of the grid view
                            field = userColumn;

                            // Attribute "source"
                            XmlAttribute source = columnNode.Attributes["source"];
                            if (source != null)
                            {
                                columnSource = source.Value;
                                userColumn.DataField = columnSource;

                                // Allow sorting
                                XmlAttribute allowSorting = columnNode.Attributes["allowsorting"];
                                if ((allowSorting == null) || ValidationHelper.GetBoolean(allowSorting.Value, true))
                                {
                                    if (columnSource.ToLower() != ExtendedBoundField.ALL_DATA.ToLower())
                                    {
                                        userColumn.SortExpression = columnSource;
                                    }
                                    else
                                    {
                                        // Sort
                                        XmlAttribute sort = columnNode.Attributes["sort"];
                                        if (sort != null)
                                        {
                                            userColumn.SortExpression = sort.Value;
                                        }
                                    }
                                }
                            }
                        }

                        field.HeaderStyle.Wrap = false;

                        // Caption
                        XmlAttribute caption = columnNode.Attributes["caption"];
                        if (caption != null)
                        {
                            columnCaption = caption.Value;
                            columnCaption = ResHelper.GetResourceName(columnCaption);
                            field.HeaderText = ResHelper.GetString(columnCaption);
                        }

                        // Width
                        XmlAttribute width = columnNode.Attributes["width"];
                        if (width != null)
                        {
                            field.ItemStyle.Width = new Unit(width.Value);
                        }

                        // Visible
                        XmlAttribute visible = columnNode.Attributes["visible"];
                        if (visible != null)
                        {
                            field.Visible = ValidationHelper.GetBoolean(visible.Value, true);
                        }

                        // Is text?
                        XmlAttribute isText = columnNode.Attributes["istext"];
                        if ((isText != null) && ValidationHelper.GetBoolean(isText.Value, false))
                        {
                            TextColumns.Add(columnSource.ToLower());
                        }

                        // Wrap?
                        XmlAttribute wrap = columnNode.Attributes["wrap"];
                        if (wrap != null)
                        {
                            if (wrap.Value.ToLower() == "false")
                            {
                                field.ItemStyle.Wrap = false;
                            }
                        }


                        // Process "filter" node
                        if (displayFilter)
                        {
                            // Filter
                            XmlNode filterNode = columnNode.SelectSingleNode("filter");
                            if (filterNode != null)
                            {
                                // Type
                                string filterType = null;
                                XmlAttribute type = filterNode.Attributes["type"];
                                if (type != null)
                                {
                                    filterType = type.Value;
                                }

                                // Source
                                string filterSource = null;
                                XmlAttribute source = filterNode.Attributes["source"];
                                filterSource = (source != null) ? source.Value : columnSource;

                                // Format
                                string filterFormat = null;
                                XmlAttribute format = filterNode.Attributes["format"];
                                if (format != null)
                                {
                                    filterFormat = format.Value;
                                }

                                // Path 
                                string filterPath = null;
                                XmlAttribute path = filterNode.Attributes["path"];
                                if (path != null)
                                {
                                    filterPath = path.Value;
                                }

                                object option = null;
                                object value = null;
                                if (FilterByQueryString)
                                {
                                    if (String.IsNullOrEmpty(filterPath))
                                    {
                                        string values = QueryHelper.GetString(columnSource, null);
                                        if (!string.IsNullOrEmpty(values))
                                        {
                                            string[] pair = values.Split(';');
                                            option = pair[0];
                                            value = pair[1];
                                        }
                                    }
                                    else
                                    {
                                        value = QueryHelper.GetString(columnSource, null);
                                    }
                                }

                                AddFilterField(filterType, filterPath, filterFormat, filterSource, ResHelper.GetString(columnCaption), filterTable, option, value);
                            }
                        }

                        // Add custom column to gridview
                        UniGridView.Columns.Add(field);
                    }
                }

                if (displayFilter)
                {
                    // Add button to the bottom of the filter table
                    showButton = new CMSButton();
                    Literal ltlBreak = new Literal();
                    TableRow tRow = new TableRow();
                    TableCell tCell = new TableCell();
                    showButton.ID = "btnShow";
                    showButton.Text = ResHelper.GetString("general.show");
                    showButton.CssClass = "ContentButton";
                    showButton.Click += ShowButton_Click;
                    showButton.EnableViewState = false;
                    ltlBreak.EnableViewState = false;
                    ltlBreak.Text = "<br /><br />";
                    tCell.Controls.Add(showButton);
                    tCell.Controls.Add(ltlBreak);
                    tCell.ColumnSpan = 2;
                    tCell.EnableViewState = false;
                    TableCell indentCell = new TableCell();
                    indentCell.EnableViewState = false;
                    indentCell.Text = "&nbsp;";
                    tRow.Cells.Add(indentCell);   // Indent 'Show' button
                    tRow.Cells.Add(tCell);
                    filterTable.Rows.Add(tRow);
                    pnlHeader.DefaultButton = showButton.ID;
                }

                // Set pagging acording to the key value "DisplayPageSizeDropdown"                
                displayPageSize = (GetKeyValue(optionNode, "DisplayPageSizeDropdown").ToLower() == "true");

                if (DataSource == null)
                {
                    // Get query information
                    XmlNode queryNode = node.SelectSingleNode("query");
                    if (queryNode != null)
                    {
                        Query = queryNode.Attributes["name"].Value;

                        // Sets the columns property if columns are defined
                        XmlAttribute columns = queryNode.Attributes["columns"];
                        if (columns != null)
                        {
                            Columns = DataHelper.GetNotEmpty(columns.Value, Columns);
                        }

                        // Load the query parameters
                        XmlNodeList parameters = queryNode.SelectNodes("parameter");
                        if ((parameters != null) && (parameters.Count > 0))
                        {
                            object[,] newParams = new object[parameters.Count, 3];
                            const int paramIndex = 0;

                            // Process all parameters
                            foreach (XmlNode param in parameters)
                            {
                                newParams[paramIndex, 0] = param.Attributes["name"].Value;
                                switch (param.Attributes["type"].Value.ToLower())
                                {
                                    case "string":
                                        newParams[paramIndex, 1] = param.Attributes["value"].Value;
                                        break;

                                    case "int":
                                        newParams[paramIndex, 1] = ValidationHelper.GetInteger(param.Attributes["value"].Value, 0);
                                        break;

                                    case "double":
                                        newParams[paramIndex, 1] = Convert.ToDouble(param.Attributes["value"].Value);
                                        break;

                                    case "bool":
                                        newParams[paramIndex, 1] = Convert.ToBoolean(param.Attributes["value"].Value);
                                        break;
                                }

                                QueryParameters = newParams;
                            }
                        }
                    }
                }

                return true;
            }
        }

        return false;
    }


    /// <summary>
    /// Reloads the grid data
    /// </summary>
    public void ReloadData()
    {
        RaiseOnBeforeDataReload();
        rowIndex = 1;
        DataSet ds = null;

        // Get Current TOP N
        if (GridView.AllowPaging && pageSizeDropdown.SelectedValue.ToUpper() != ALL)
        {
            int currentPageIndex = UniGridView.PageIndex;
            if (!regularPaging)
            {
                currentPageIndex = currentPageIndex - (currentPageIndex % 10);
            }

            int pageSize = ValidationHelper.GetInteger(pageSizeDropdown.SelectedValue, 0);
            if (pageSize > 0)
            {
                CurrentTopN = pageSize * (currentPageIndex + 1 + UniGridView.PagerSettings.PageButtonCount);
            }
            else
            {
                CurrentTopN = UniGridView.PageSize * (currentPageIndex + 1 + UniGridView.PagerSettings.PageButtonCount);
            }
        }

        if (CurrentTopN < TopN)
        {
            CurrentTopN = TopN;
        }

        // If datasource for unigrid is query (not dataset), then execute query
        if (!string.IsNullOrEmpty(Query))
        {
            // Reload the data with current parameters
            GeneralConnection con = ConnectionHelper.GetConnection();

            // Set order by
            string currentOrder = OrderBy;
            if (!String.IsNullOrEmpty(SortDirect))
            {
                currentOrder = SortDirect;
            }

            ds = con.ExecuteQuery(Query, QueryParameters, CompleteWhereCondition, currentOrder, CurrentTopN, Columns);
        }
        // External dataset is used
        else
        {
            ds = RaiseDataReload();
        }

        if (DataHelper.DataSourceIsEmpty(ds))
        {
            ds = new DataSet();
            ds.Tables.Add();
            UniGridView.DataSource = ds;
        }
        else
        {
            UniGridView.DataSource = ds;
            // Sort unigrid data source if external DataSet is used
            if (string.IsNullOrEmpty(Query) && !DataSourceIsSorted)
            {
                SortUniGridDataSource();
            }
        }

        SetUnigridControls();
        UniGridView.DataBind();

        // Get rows count
        if (UniGridView.DataSource is DataSet)
        {
            mRowsCount = ((DataSet)UniGridView.DataSource).Tables[0].Rows.Count;
        }
        else if (UniGridView.DataSource is DataTable)
        {
            mRowsCount = ((DataTable)UniGridView.DataSource).Rows.Count;
        }
        else if (UniGridView.DataSource is DataView)
        {
            mRowsCount = ((DataView)UniGridView.DataSource).Table.Rows.Count;
        }

        if ((displayFilter) && (FilterLimit > 0))
        {
            if (FilterByQueryString)
            {
                // Hide filters if rowcount is lower than limit
                if (!FilterIsSet && ((RowsCount <= FilterLimit) || (RowsCount == 0)))
                {
                    // Set filter visibility
                    filter.Visible = false;
                    pnlFooter.Visible = false;
                }
            }
            else
            {
                if (!RequestHelper.IsPostBack() || !FilterIsSet)
                {
                    // Hide filters if rowcount is lower than limit
                    if ((RowsCount <= FilterLimit) || (RowsCount == 0))
                    {
                        // Set filter visibility
                        filter.Visible = false;
                        pnlFooter.Visible = false;
                    }
                }
            }
        }

        if (RowsCount == 0)
        {
            pnlFooter.Visible = false;
        }
    }


    /// <summary>
    /// Returns attribute value of the specific option (specific key node)
    /// </summary>
    /// <param name="optionNode">option node containing key nodes</param>
    /// <param name="keyName">Attribute name of the specific key node</param>
    /// <returns>Attribute value of the specific key node</returns>
    protected static string GetKeyValue(XmlNode optionNode, string keyName)
    {
        string keyValue = "";

        if (optionNode != null)
        {
            XmlNode keyNode = optionNode.SelectSingleNode("key[@name='" + keyName + "']");
            if (keyNode != null)
            {
                keyValue = keyNode.Attributes["value"].Value;
            }
        }

        return keyValue;
    }


    /// <summary>
    /// Add filter field to the filter table
    /// </summary>
    /// <param name="fieldType">Field type</param>
    /// <param name="fieldPath">Filter contol file path. If not starts with ~/ default directory '~/CMSAdminControls/UI/UniGrid/Filters/' will be inserted at begining</param>
    /// <param name="filterFormat">Filter format string</param>
    /// <param name="fieldSourceName">Source field name</param>
    /// <param name="fieldDisplayName">Field display name</param>
    /// <param name="filterTable">Filter table</param>
    /// <param name="filterOption">Filter option</param>
    /// <param name="filterValue">Filter value</param>
    protected void AddFilterField(string fieldType, string fieldPath, string filterFormat, string fieldSourceName, string fieldDisplayName, Table filterTable, object filterOption, object filterValue)
    {
        TableRow tRow = new TableRow();

        TableCell tCellName = new TableCell();
        TableCell tCellOption = new TableCell();
        TableCell tCellValue = new TableCell();

        // Ensure fieldSourceName is Javascript valid
        fieldSourceName = fieldSourceName.Replace(ALL, "__ALL__");

        // Label
        Label textName = new Label();
        textName.Text = fieldDisplayName + ":";
        textName.ID = fieldSourceName + "Name";
        textName.EnableViewState = false;
        tCellName.Controls.Add(textName);
        tRow.Cells.Add(tCellName);

        // Filter option
        string option = null;
        if (filterOption != null)
        {
            option = ValidationHelper.GetString(filterOption, null);
        }

        // Filter value
        string value = null;
        if (filterValue != null)
        {
            value = ValidationHelper.GetString(filterValue, null);
        }

        // Filter definition
        // 0 = Type
        // 1 = Format
        // 2 = Option control
        // 3 = Filter value control
        // 4 = FilterID
        object[] filterDefinition = new object[5];
        filterDefinition[0] = fieldType;
        filterDefinition[1] = filterFormat;

        // If no filter path us default filter
        if (String.IsNullOrEmpty(fieldPath))
        {
            switch (fieldType)
            {
                // Text filter
                case "text":
                    {
                        DropDownList textOptionFilterField = new DropDownList();
                        textOptionFilterField.Items.Add(new ListItem("LIKE", "LIKE"));
                        textOptionFilterField.Items.Add(new ListItem("NOT LIKE", "NOT LIKE"));
                        textOptionFilterField.Items.Add(new ListItem("=", "="));
                        textOptionFilterField.Items.Add(new ListItem("<>", "<>"));
                        textOptionFilterField.CssClass = "ContentDropdown";
                        textOptionFilterField.ID = fieldSourceName;

                        // Select filter option
                        try
                        {
                            textOptionFilterField.SelectedValue = option;
                        }
                        catch
                        {
                        }

                        tCellOption.Controls.Add(textOptionFilterField);
                        tRow.Cells.Add(tCellOption);

                        // Add text field
                        TextBox textValueFilterField = new TextBox();
                        textValueFilterField.ID = fieldSourceName + "TextValue";
                        textValueFilterField.Text = value;
                        tCellValue.Controls.Add(textValueFilterField);
                        tRow.Cells.Add(tCellValue);

                        filterDefinition[2] = textOptionFilterField;
                        filterDefinition[3] = textValueFilterField;
                    }
                    break;

                // Boolean filter
                case "bool":
                    {
                        DropDownList booleanOptionFilterField = new DropDownList();
                        booleanOptionFilterField.Items.Add(new ListItem(ResHelper.GetString("general.selectall"), ""));
                        booleanOptionFilterField.Items.Add(new ListItem(ResHelper.GetString("general.yes"), "1"));
                        booleanOptionFilterField.Items.Add(new ListItem(ResHelper.GetString("general.no"), "0"));
                        booleanOptionFilterField.CssClass = "ContentDropdown";
                        booleanOptionFilterField.ID = fieldSourceName;

                        // Select filter option
                        try
                        {
                            booleanOptionFilterField.SelectedValue = value;
                        }
                        catch
                        {
                        }

                        tCellValue.Controls.Add(booleanOptionFilterField);
                        tRow.Cells.Add(tCellValue);

                        filterDefinition[3] = booleanOptionFilterField;
                    }
                    break;

                // Integer filter
                case "integer":
                case "double":
                    {
                        DropDownList numberOptionFilterField = new DropDownList();
                        numberOptionFilterField.Items.Add(new ListItem("=", "="));
                        numberOptionFilterField.Items.Add(new ListItem("<>", "<>"));
                        numberOptionFilterField.Items.Add(new ListItem("<", "<"));
                        numberOptionFilterField.Items.Add(new ListItem(">", ">"));
                        numberOptionFilterField.ID = fieldSourceName;

                        // Select filter option
                        try
                        {
                            numberOptionFilterField.SelectedValue = option;
                        }
                        catch
                        {
                        }

                        // Add filter field
                        tCellOption.Controls.Add(numberOptionFilterField);
                        tRow.Cells.Add(tCellOption);

                        TextBox numberValueFilterField = new TextBox();
                        numberValueFilterField.ID = fieldSourceName + "NumberValue";
                        numberValueFilterField.Text = value;
                        numberValueFilterField.EnableViewState = false;

                        RangeValidator numberValidator = new RangeValidator();
                        numberValidator.ID = "numberValidator" + numberValueFilterField.ID;
                        numberValidator.Type = (fieldType == "integer") ? ValidationDataType.Integer : ValidationDataType.Double;
                        numberValidator.MinimumValue = "0";
                        numberValidator.MaximumValue = "1000";
                        numberValidator.ControlToValidate = numberValueFilterField.ID;
                        numberValidator.ErrorMessage = "Number type must be " + fieldType + " less than " + numberValidator.MaximumValue + "!";
                        numberValidator.EnableViewState = false;

                        tCellValue.Controls.Add(numberValueFilterField);
                        tCellValue.Controls.Add(numberValidator);
                        tRow.Cells.Add(tCellValue);

                        filterDefinition[2] = numberOptionFilterField;
                        filterDefinition[3] = numberValueFilterField;
                    }
                    break;
            }
        }
        // Else if filter path is defined use custom filter
        else
        {
            string path = fieldPath.StartsWith("~/") ? fieldPath : FilterDirectoryPath + fieldPath.TrimStart('/');
            CMSAbstractBaseFilterControl filterControl = LoadControl(path) as CMSAbstractBaseFilterControl;

            filterControl.ID = fieldSourceName;
            tCellValue.Controls.Add(filterControl);
            tCellValue.Attributes["colspan"] = "2";
            tRow.Cells.Add(tCellValue);
            filterControl.FilteredControl = this;
            if (!RequestHelper.IsPostBack())
            {
                filterControl.Value = value;
            }

            filterDefinition[3] = filterControl;
            filterDefinition[4] = filterControl.ID;
        }

        mFilterFields.Add(filterDefinition);

        filterTable.Rows.Add(tRow);
    }


    /// <summary>
    /// Set filter to the grid view and save it to the view state
    /// </summary>
    /// <param name="reloadData">Reload data</param>
    protected void SetFilter(bool reloadData)
    {
        string where = GetFilter();

        // Filter by query string
        if (FilterByQueryString && !reloadData)
        {
            string url = URLRewriter.CurrentURL;
            foreach (string name in queryStringHashTable.Keys)
            {
                if (queryStringHashTable[name] != null)
                {
                    string value = HttpContext.Current.Server.UrlEncode(queryStringHashTable[name].ToString());
                    url = UrlHelper.AddParameterToUrl(url, name, value);
                }
            }
            UrlHelper.Redirect(url);
        }
        else
        {
            WhereClause = where;
            FilterIsSet = true;
            if ((!DelayedReload) && (reloadData))
            {
                // Get data from database and set them to the grid view
                ReloadData();
            }
        }
    }


    /// <summary>
    /// Returns where condition from unigrid filters
    /// </summary>
    public string GetFilter()
    {
        string where = string.Empty;
        // Count of the conditions in the 'where clause'
        int whereConditionCount = 0;

        // Process all filter fields
        foreach (object[] filter in mFilterFields)
        {
            string filterFormat = (string)filter[1];
            // AND in 'where clause'  
            string andExpression;
            Control mainControl = (Control)filter[2];
            Control valueControl = (Control)filter[3];
            string filterPath = (string)filter[4];

            if (String.IsNullOrEmpty(filterPath))
            {
                if (mainControl is DropDownList)
                {
                    // Dropdown list filter
                    DropDownList ddlistControl = (DropDownList)mainControl;
                    TextBox txtControl = (TextBox)valueControl;

                    string textboxValue = txtControl.Text;
                    string textboxID = txtControl.ID;

                    // Empty field -> no filter is set for this field
                    if (textboxValue != "")
                    {
                        string op = ddlistControl.SelectedValue;
                        string value = textboxValue.Replace("\'", "''");
                        string columnName = ddlistControl.ID;

                        // Format {0} = column name, {1} = operator, {2} = value, {3} = default condition
                        string defaultFormat = null;

                        if (textboxID.EndsWith("TextValue"))
                        {
                            switch (op.ToLower())
                            {
                                // LIKE operators
                                case "like":
                                case "not like":
                                    defaultFormat = "{0} {1} N'%{2}%'";
                                    break;

                                // Standard operators
                                default:
                                    defaultFormat = "{0} {1} N'{2}'";
                                    break;
                            }
                        }
                        else // textboxID.EndsWith("NumberValue")
                        {
                            defaultFormat = "{0} {1} {2}";
                        }

                        string defaultCondition = String.Format(defaultFormat, columnName, op, value);

                        string condition = defaultCondition;
                        if (filterFormat != null)
                        {
                            condition = String.Format(filterFormat, columnName, op, value, defaultCondition);
                        }

                        andExpression = (whereConditionCount > 0 ? " AND " : string.Empty);

                        // ddlistControl.ID                 - column name
                        // ddlistControl.SelectedValue      - condition option
                        // textboxSqlValue                  - condition value                        
                        where += andExpression + "(" + condition + ")";
                        whereConditionCount++;
                    }

                    // Prepare query string
                    if (FilterByQueryString)
                    {
                        queryStringHashTable[ddlistControl.ID] = ddlistControl.SelectedValue + ";" + textboxValue;
                    }
                }
                else if (valueControl is DropDownList)
                {
                    // Checkbox filter
                    DropDownList currentControl = (DropDownList)valueControl;
                    string value = currentControl.SelectedValue;
                    if (value != "")
                    {
                        andExpression = (whereConditionCount > 0) ? " AND" : "";
                        where += andExpression + " " + currentControl.ID + " = " + value;
                        whereConditionCount++;
                    }

                    // Prepare query string
                    if (FilterByQueryString)
                    {
                        queryStringHashTable[currentControl.ID] = ";" + value;
                    }
                }
            }
            // If is defined filter path
            else
            {
                CMSAbstractBaseFilterControl customFilter = (CMSAbstractBaseFilterControl)filter[3];
                string customWhere = customFilter.WhereCondition;
                if (!String.IsNullOrEmpty(customWhere))
                {
                    andExpression = (whereConditionCount > 0) ? " AND " : "";
                    where += andExpression + customWhere;
                    whereConditionCount++;
                }

                // Prepare query string
                if (FilterByQueryString && RequestHelper.IsPostBack())
                {
                    queryStringHashTable[customFilter.ID] = customFilter.Value;
                }
            }
        }

        return where;
    }


    /// <summary>
    /// OnShowButtonClick event handler
    /// </summary>
    public event EventHandler OnShowButtonClick;


    /// <summary>
    /// Process data from filter
    /// </summary>
    protected void ShowButton_Click(object sender, EventArgs e)
    {
        if (OnShowButtonClick != null)
        {
            OnShowButtonClick(sender, e);
        }

        SetFilter(!DelayedReload);
    }


    /// <summary>
    /// Set page size dropdown list according to PageSize property
    /// </summary>
    private void SetPageSize()
    {
        if (pageSizeDropdown.Items.Count == 0)
        {
            string[] sizes = PageSize.Split(',');
            pageSizeDropdown.Items.Clear();
            foreach (string size in sizes)
            {
                if (size.ToUpper() != ALL)
                {
                    pageSizeDropdown.Items.Add(new ListItem(size));
                }
                else
                {
                    pageSizeDropdown.Items.Add(new ListItem(ResHelper.GetString("general.selectall"), ALL));
                }
            }
        }
    }


    protected void pageSizeDropdown_SelectedIndexChanged(object sender, EventArgs e)
    {
        RaisePageSizeChanged();
    }


    protected void UniGridView_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        RaiseBeforePageIndexChanging(sender, e);

        // Reset row index for setting odd and even rows
        rowIndex = 1;

        // Set page index
        UniGridView.PageIndex = e.NewPageIndex;
        UniGridView.DataBind();
    }


    protected void UniGridView_Sorting(object sender, EventArgs e)
    {
        RaiseBeforeSorting(sender, e);
    }


    protected void UniGridView_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        // No action is required
    }


    /// <summary>
    /// OnBefore page index changing
    /// </summary>
    void UniGridView_OnBeforePageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        UniGridView.PageIndex = e.NewPageIndex - (e.NewPageIndex % 10);

        regularPaging = true;

        ReloadData();
    }


    /// <summary>
    /// After data bound event
    /// </summary>
    protected void UniGridView_DataBound(object sender, EventArgs e)
    {
        // Set actions hash into hidden field
        SetActionsHash();
    }


    /// <summary>
    /// Sorts UniGrid data source according to sort directive saved in viewstate
    /// </summary>
    protected void SortUniGridDataSource()
    {
        if (SortDirect != "")
        {
            // If source isn't empty
            if (!DataHelper.DataSourceIsEmpty(UniGridView.DataSource))
            {
                // Set sort directive from viewstate
                if (UniGridView.DataSource is DataTable)
                {
                    try
                    {
                        ((DataTable)(UniGridView.DataSource)).DefaultView.Sort = SortDirect;
                    }
                    catch { }
                }
                else if (UniGridView.DataSource is DataSet)
                {
                    try
                    {
                        ((DataSet)(UniGridView.DataSource)).Tables[0].DefaultView.Sort = SortDirect;
                        UniGridView.DataSource = ((DataSet)(UniGridView.DataSource)).Tables[0].DefaultView;
                    }
                    catch { }
                }
            }
        }
    }


    /// <summary>
    /// Changes sorting direction by specified column
    /// </summary>
    /// <param name="orderByColumn">Column name to order by.</param>
    /// <param name="orderByString">Old order by string.</param> 
    protected void ChangeSortDirection(string orderByColumn, string orderByString)
    {
        orderByColumn = orderByColumn.ToLower().Trim();
        orderByString = orderByString.ToLower().Trim();

        // If order by column is long text use CAST in ORDER BY part of query
        if (TextColumns.Contains(orderByColumn))
        {
            if (orderByString.ToLower().StartsWith("cast(" + orderByColumn))
            {
                if (orderByString.EndsWith("desc"))
                {
                    SortDirect = "CAST(" + orderByColumn + " AS nvarchar(32)) asc";
                }
                else
                {
                    SortDirect = "CAST(" + orderByColumn + " AS nvarchar(32)) desc";
                }
            }
            else
            {
                SortDirect = "CAST(" + orderByColumn + " AS nvarchar(32)) asc";
            }
        }
        else
        {
            // Sort by the same column -> the other directon
            if (orderByString.StartsWith(orderByColumn.ToLower()))
            {
                SortDirect = (orderByString.EndsWith("desc")) ? orderByColumn + " asc" : orderByColumn + " desc";
            }
            // Sort by a new column -> implicitly direction is ASC
            else
            {
                SortDirect = orderByColumn + " asc";
            }
        }
    }


    /// <summary>
    /// Sets unigrid controls.
    /// </summary>
    protected void SetUnigridControls()
    {
        if (!RequestHelper.IsPostBack())
        {
            pnlFooter.Visible = false;
        }
        filter.Visible = displayFilter;

        // Indicates whether unigrid datasource is empty or not
        isEmpty = DataHelper.DataSourceIsEmpty(UniGridView.DataSource);

        if (HideControlForZeroRows && isEmpty && (WhereClause == ""))
        {
            // Hide filter
            filter.Visible = false;
        }
        else
        {
            if (!isEmpty && displayPageSize)
            {
                // Display pagging dropdown
                pnlFooter.Visible = true;

                if (pageSizeDropdown.SelectedValue.ToUpper() == ALL)
                {
                    UniGridView.AllowPaging = false;
                }
                else
                {
                    int pageSize = ValidationHelper.GetInteger(pageSizeDropdown.SelectedValue, 25);
                    if (pageSize > 0)
                    {
                        UniGridView.AllowPaging = true;
                        UniGridView.PageSize = pageSize;
                    }
                }
            }
        }
    }


    protected void UniGridView_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
        // Deleting is handled in custom method
    }


    protected void UniGridView_RowEditing(object sender, GridViewEditEventArgs e)
    {
        // Editing is handled in custom method
    }


    protected void UniGridView_RowUpdating(object sender, GridViewUpdateEventArgs e)
    {
        // Updating is handled in custom method
    }


    protected void UniGridView_RowCreating(object sender, GridViewRowEventArgs e)
    {
        // If row type is header
        if (e.Row.RowType == DataControlRowType.Header)
        {
            foreach (TableCell Cell in e.Row.Cells)
            {
                // If there is some sorting expression
                DataControlFieldCell dataField = (DataControlFieldCell)Cell;
                if (!DataHelper.IsEmpty(dataField.ContainingField.SortExpression))
                {
                    string sort = SortDirect.ToLower().Trim();
                    if (sort.StartsWith("cast("))
                    {
                        sort = sort.Substring(5);
                    }
                    // Get column name from sort expression
                    int space = sort.IndexOf(' ');
                    if (space > -1)
                    {
                        sort = sort.Substring(0, space);
                    }
                    // If actual sorting expressions is this cell
                    if (String.Equals(sort, dataField.ContainingField.SortExpression, StringComparison.InvariantCultureIgnoreCase))
                    {
                        // Initialize sort arrow
                        Image sortArrow = new Image();
                        sortArrow.ID = "imgSort_" + Guid.NewGuid();
                        sortArrow.EnableViewState = false;
                        sortArrow.AlternateText = ResHelper.GetString("general.sort");
                        sortArrow.ImageUrl = SortDirect.ToLower().Trim().EndsWith("desc") ? GetActionImage("SortDown.png") : GetActionImage("SortUp.png");

                        if (DataHelper.IsEmpty(Cell.Text))
                        {
                            if ((Cell.Controls.Count != 0) && (Cell.Controls[0] != null))
                            {
                                // Add original text
                                Cell.Controls[0].Controls.Add(new LiteralControl(((LinkButton)(Cell.Controls[0])).Text));
                                // Add one space before image
                                Cell.Controls[0].Controls.Add(new LiteralControl("&nbsp;"));
                                Cell.Controls[0].Controls.Add(sortArrow);
                            }
                            else
                            {
                                // Add one space before image
                                Cell.Controls.Add(new LiteralControl("&nbsp;"));
                                Cell.Controls.Add(sortArrow);
                            }
                        }
                    }
                }
            }
        }
        else if (e.Row.RowType == DataControlRowType.DataRow)
        {
            e.Row.CssClass = (rowIndex % 2 == 0) ? "OddRow" : "EvenRow";
            rowIndex++;
        }
        else if (e.Row.RowType == DataControlRowType.Footer)
        {
            e.Row.CssClass = "UniGridFooter";
        }
        else if (e.Row.RowType == DataControlRowType.Pager)
        {
            e.Row.CssClass = "UniGridPager";
        }
    }


    /// <summary>
    /// Return List of selected Items.
    /// </summary>
    private ArrayList GetSelectedItems()
    {
        return GetHiddenValues(hidSelection);
    }


    /// <summary>
    /// Sets selection values for UniGrid.
    /// </summary>
    /// <param name="values">Arraylist of values to selection</param>
    private void SetSectedItems(ArrayList values)
    {
        SetHiddenValues(values, hidSelection);
    }


    /// <summary>
    /// Return List of deselected Items.
    /// </summary>
    private ArrayList GetDeselectedItems()
    {
        return GetHiddenValues(hidDeSelection);
    }


    /// <summary>
    /// Return List of newly selected Items.
    /// </summary>
    private ArrayList GetNewlySelectedItems()
    {
        return GetHiddenValues(hidNewSelection);
    }


    /// <summary>
    /// Return array list from hidden field 
    /// </summary>
    /// <param name="field">Hidden field with values separated with |</param>
    private static ArrayList GetHiddenValues(HiddenField field)
    {
        string hiddenValue = field.Value.Trim('|');
        ArrayList list = new ArrayList();
        string[] values = hiddenValue.Split('|');
        foreach (string value in values)
        {
            if (!list.Contains(value))
            {
                list.Add(value);
            }
        }
        list.Remove("");
        return list;
    }


    /// <summary>
    /// Sets array list values into hidden field.
    /// </summary>
    /// <param name="values">Arraylist of values to selection</param>
    /// <param name="field">Hidden field</param>
    private static void SetHiddenValues(ArrayList values, HiddenField field)
    {
        if ((field != null) && (values != null))
        {
            StringBuilder sb = new StringBuilder();
            foreach (string value in values)
            {
                sb.Append(value + "|");
            }
            field.Value = "|" + sb;
        }
    }


    /// <summary>
    /// Sets hidden field with actions hashes.
    /// </summary>
    private void SetActionsHash()
    {
        if (ActionsID.Count > 0)
        {
            SetHiddenValues(ActionsID, hidActionsHash);
        }
    }


    /// <summary>
    /// Uncheck all checkboxes in selection column
    /// </summary>
    public void ResetSelection()
    {
        hidSelection.Value = String.Empty;
        hidNewSelection.Value = String.Empty;
        hidDeSelection.Value = String.Empty;
        resetSelection = true;
    }


    private void RegisterCmdScripts()
    {
        StringBuilder builder = new StringBuilder();

        // Actions
        builder.Append("function SetUniGridCmd_" + ClientID + "(name, arg){\n");
        builder.Append("var nameObj = document.getElementById('" + hidCmdName.ClientID + "');\n");
        builder.Append("var argObj = document.getElementById('" + hidCmdArg.ClientID + "');\n");
        builder.Append("if ((nameObj != null)&&(argObj != null)){\n");
        builder.Append("nameObj.value = name;\n");
        builder.Append("argObj.value = arg;\n");
        builder.Append(Page.ClientScript.GetPostBackEventReference(this, "UniGridAction"));
        builder.Append(";} return false;\n");
        builder.Append("}\n");

        // Selection - click
        builder.Append("function SelectionClick_" + ClientID + "(checkBox, arg){\n");
        builder.Append("if (checkBox != null){\n");
        builder.Append("    var selectionObj = document.getElementById('" + hidSelection.ClientID + "');\n");
        builder.Append("    var newSelectionObj = document.getElementById('" + hidNewSelection.ClientID + "');\n");
        builder.Append("    var deSelectionObj = document.getElementById('" + hidDeSelection.ClientID + "');\n");
        builder.Append("    if ((selectionObj != null)&&(newSelectionObj != null)&&(deSelectionObj != null)){\n");
        builder.Append("        if (newSelectionObj.value == '') {\n");
        builder.Append("            newSelectionObj.value = '|';\n");
        builder.Append("        }\n");
        builder.Append("        if (deSelectionObj.value == '') {\n");
        builder.Append("            deSelectionObj.value = '|';\n");
        builder.Append("        }\n");
        builder.Append("        if (selectionObj.value == '') {\n");
        builder.Append("            selectionObj.value = '|';\n");
        builder.Append("        }\n");
        builder.Append("        if (checkBox.checked) {\n");
        builder.Append("            selectionObj.value += arg + '|';\n");
        builder.Append("            if (deSelectionObj.value.indexOf('|' + arg + '|') >= 0) {\n");
        builder.Append("                deSelectionObj.value = deSelectionObj.value.replace('|' + arg + '|', '|');\n");
        builder.Append("            }\n");
        builder.Append("            else {\n");
        builder.Append("                newSelectionObj.value += arg + '|';\n");
        builder.Append("            }\n");
        builder.Append("        }\n");
        builder.Append("        else {\n");
        builder.Append("            selectionObj.value = selectionObj.value.replace('|' + arg + '|', '|');\n");
        builder.Append("            if (newSelectionObj.value.indexOf('|' + arg + '|') >= 0) {\n");
        builder.Append("                newSelectionObj.value = newSelectionObj.value.replace('|' + arg + '|', '|');\n");
        builder.Append("            }\n");
        builder.Append("            else {\n");
        builder.Append("                deSelectionObj.value += arg + '|';\n");
        builder.Append("            }\n");
        builder.Append("        }\n");
        builder.Append("    }\n");
        builder.Append("}\n");
        builder.Append("}\n");

        // Selection - select all
        builder.Append("function SelectAllClick_" + ClientID + "(checkBox){\n");
        builder.Append("var elements = document.getElementsByTagName(\"INPUT\");\n");
        builder.Append("var regularExpresssion = new RegExp('" + ClientID + "');\n");
        builder.Append("for(i=0; i<elements.length;i++){\n");
        builder.Append("    if(elements[i].type == 'checkbox'){\n");
        builder.Append("        if(elements[i].id.match(regularExpresssion)) {\n");
        builder.Append("            if((!elements[i].id != checkBox.id) && (checkBox.checked != elements[i].checked)) {\n");
        builder.Append("                elements[i].click();\n");
        builder.Append("            }\n");
        builder.Append("        }\n");
        builder.Append("    }\n");
        builder.Append("}");
        builder.Append("}");

        // Selection - clear
        builder.Append("function ClearSelection_" + ClientID + "() {\n");
        builder.Append(" var inputs = document.getElementsByTagName('input');\n");
        builder.Append(" if (inputs != null) {\n");
        builder.Append("     for (var i = 0; i< inputs.length; i++) {\n");
        builder.Append("         if ((inputs[i].type.toLowerCase() == 'checkbox') && (inputs[i].id.match(/^" + ClientID + "/i))) {\n");
        builder.Append("             inputs[i].checked = false;\n");
        builder.Append("         }\n");
        builder.Append("     }\n");
        builder.Append(" }\n");
        builder.Append(" var hidSelection = document.getElementById('" + hidSelection.ClientID + "');\n");
        builder.Append(" var hidNewSelection = document.getElementById('" + hidNewSelection.ClientID + "');\n");
        builder.Append(" var hidDeSelection = document.getElementById('" + hidDeSelection.ClientID + "');\n");
        builder.Append(" if (hidSelection != null) {\n");
        builder.Append("     hidSelection.value = '';\n");
        builder.Append(" }\n");
        builder.Append(" if (hidNewSelection != null) {\n");
        builder.Append("     hidNewSelection.value = '';\n");
        builder.Append(" }\n");
        builder.Append(" if (hidDeSelection != null) {\n");
        builder.Append("     hidDeSelection.value = '';\n");
        builder.Append(" }\n");
        builder.Append("}\n");

        if (resetSelection)
        {
            builder.Append("if (typeof(ClearSelection_" + ClientID + ") != 'undefined')\n");
            builder.Append("{\n");
            builder.Append("    ClearSelection_" + ClientID + "();\n");
            builder.Append("}\n");
        }

        ScriptHelper.RegisterStartupScript(this, typeof(string), "SetUniGridCmd_" + ClientID, ScriptHelper.GetScript(builder.ToString()));
    }

    #endregion
}
